use std::{
    net::SocketAddr,
    sync::{Arc, Mutex},
    convert::Infallible,
};

use hyper::{
    service::{make_service_fn, service_fn},
    header::HeaderValue,
    Body, Request, Server, Response,
};

use serde::{Deserialize, Serialize};
use serde_with::serde_as;
use serde_json;

use lox::{BridgeDb, OPENINV_LENGTH};

#[serde_as]
#[derive(Serialize, Deserialize)]
pub struct Invite {
    #[serde_as(as = "[_; OPENINV_LENGTH]")]
    invite: [u8; OPENINV_LENGTH],
}

#[tokio::main]
async fn main() {

    // Create and initialize a new db
    let mut bridgedb = BridgeDb::new();
        for i in &[1u32, 5, 7, 12, 19, 20, 22] {
        bridgedb.insert_openinv(*i);
    }

    let db = Arc::new(Mutex::new(bridgedb));

    let new_service = make_service_fn(move |_conn| {
        let db = db.clone();
        async move {
            Ok::<_, Infallible>(service_fn(move |_req: Request<Body>| {
                let db = db.clone();
                async move {
                    Ok::<_, Infallible>(generate_invite(db))
                }
            }))
        }
    });

    let addr = SocketAddr::from(([127, 0, 0, 1], 8001));
    let server = Server::bind(&addr).serve(new_service);
    let graceful = server.with_graceful_shutdown(shutdown_signal());
    println!("Listening on {}", addr);

    if let Err(e) = graceful.await {
        eprintln!("server error: {}", e);
    }
}

async fn shutdown_signal() {
    tokio::signal::ctrl_c()
        .await
        .expect("failed to listen for ctrl+c signal");
}

fn generate_invite(db: Arc<Mutex<lox::BridgeDb>>) -> Response<Body> {
    let obj = db.lock().unwrap();
    let invite = Invite {
        invite: obj.invite(),
    };
    let token = serde_json::to_string(&invite).unwrap();
    let mut resp = Response::new(Body::from(token));
    resp.headers_mut().insert("Access-Control-Allow-Origin", HeaderValue::from_static("*"));
    resp
}
